# -*- coding: utf-8 -*-
import os

TESTS_PATH = os.path.realpath(os.path.dirname(__file__))
PROJECT_PATH = os.path.realpath(os.path.join(TESTS_PATH, '..'))
SAMPLE_DATA = os.path.join(TESTS_PATH, 'sample_data')
